from IPython.display import display, clear_output
import numpy as np
import matplotlib.pyplot as plt
import time

# We create a function to visualize the data for a given batch


def add_label(k, ax, x0, y0, predictions, labels):

    months = ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"]

    if labels is not None:
        ax.text(x0-5, y0, months[labels[k].item()], fontsize=30)
        if predictions is not None:
            if predictions[k] == labels[k]:
                color = "green"
            else:
                color = "red"
            ax.text(x0-5, y0+15, months[predictions[k].item()], fontsize=30,
                    bbox=dict(facecolor=color, alpha=0.5))

    return ax


def plot_data(data, labels, predictions=None, n_cols=4, figsize=(15,15), axid=None, delay=10, cmap="turbo", coastlines=False):

    data_shape = data.shape

    if axid is None:
        n_rows = int( np.ceil( data_shape[0] // n_cols ))
        k = 0
    else:
        n_rows, n_cols = 1, 1
        k = axid % data_shape[0]
        axid += 1

    x0, y0 = data_shape[2] // 2 - 3, data_shape[3] // 2

    fig, axes = plt.subplots(nrows=n_rows, ncols=n_cols, figsize=figsize, squeeze=False)

    vmin, vmax =  data.min(), data.max()

    if coastlines:
        img = plt.imread(".coastlines.png")

    for i in range(n_rows):
        for j in range(n_cols):
            im = axes[i, j].imshow(np.flip(data[k][0].cpu().numpy(), 0), vmin=vmin, vmax=vmax, cmap=cmap)
            axes[i, j].axes.get_xaxis().set_ticks([])
            axes[i, j].axes.get_yaxis().set_ticks([])
            if coastlines:
                axes[i, j].imshow(img, extent=im.get_extent())
            if axid is None:
                axes[i, j] = add_label(k, axes[i, j], x0, y0, predictions, labels)
                k += 1

    cbs = fig.colorbar(im, ax=axes, fraction=0.04, pad=0.04)

    if axid is not None:
        display(fig)
        clear_output(wait=True)
        time.sleep(delay)
        axes[0, 0] = add_label(k, axes[0, 0], x0, y0, predictions, labels)

    return axid
